//---------------------------------------------------------------------------


#pragma hdrstop

#include "canvasPainter.h"


//---------------------------------------------------------------------------
GcanvasPainter::GcanvasPainter(GgraphicsBitmap *acanvas){
    canvas = acanvas;
}

GgraphicsBitmap * GcanvasPainter::getBitmap(){
  return canvas;
}

int GcanvasPainter::drawRect(Grect &rect, Gcolor &color){
           Grect newRect = rect;
           newRect.move(*(Gpoint*)canvas);
           newRect &= *(Grect*)canvas;
          if (newRect.isEmpty()) return 0;
          int *linearBuffer = canvas->getLine(rect.top()) + rect.left();
          int newColor = color.valColor();
          for (int i=0;i<newRect.height();i++){
          for (int j=0;j<newRect.width();j++)
              linearBuffer[j] = newColor;
           linearBuffer += canvas->width();
          }
     return 0;
}

int GcanvasPainter::drawRectGradient(Grect &rect,  Gcolor &color, Gcolor &color1){
          Grect newRect = rect;
          newRect.move(*(Gpoint*)canvas);
          newRect &= *(Grect*)canvas;
          if (newRect.isEmpty()) return 0;
          int g,b, r  ;
          int *linearBuffer = canvas->getLine(rect.top()) + rect.left();
          int newColor;
          float c=   color.red();
          float d =  color.green();
          float e =  color.blue();
          float r1 = (float)(color1.red() - color.red())/(float)newRect.height();
          float g1 = (float)(color1.green() - color.green())/(float)newRect.height();
          float b1 = (float)(color1.blue() - color.blue())/(float)newRect.height();
          
          for (int i=0;i<newRect.height();i++){
              r = (u8)(c + r1*i);
              g = (u8)(d + g1*i);
              b = (u8)(e + b1*i);
             newColor = Gcolor(r,g,b).valColor();

          for (int j=0;j<newRect.width();j++)
               linearBuffer[j] = newColor;
           linearBuffer += canvas->width();
          }
     return 0;
}

int GcanvasPainter::borderColor(int width,int padding, Gcolor color){
     drawRect(Grect(0,0,canvas->width(),width),color);
    drawRect(Grect(0,0,width,canvas->height()),color);
    drawRect(Grect(0,canvas->height() - width,canvas->width(),width),color);
    drawRect(Grect(canvas->width()-width,0,width,canvas->height()),color);
    return 0;
}

int GcanvasPainter::drawBitmap(GgraphicsBitmap *bitmap, Grect rect, int x, int y ){
      if (rect.isEmpty()) {/*ShowMessage("Error");*/return 0;}

      int *linearBuffer = canvas->getLine(rect.top()) + rect.left();
      int *linerBufferOrigen = bitmap->getLine(y) + x;
      for (int i=0;i<rect.height();i++){
        for (int j=0;j<rect.width();j++)
                  linearBuffer[j] = linerBufferOrigen[j];
           linearBuffer += canvas->width();
           linerBufferOrigen += bitmap->width();
      }
      return 1;
 }

 int GcanvasPainter::drawBitmapTransparent(GgraphicsBitmap *bitmap, Grect rect, Gcolor &color){
      if (rect.isEmpty()) {ShowMessage("Error");return 0;}
      int *linearBuffer = canvas->getLine(rect.top()) + rect.left();
      int *linerBufferOrigen = bitmap->getLine(0);
            int valColor =  color.valColor();
      for (int i=0;i<rect.height();i++){
        for (int j=0;j<rect.width();j++)
             if  (linerBufferOrigen[j]  != valColor )
                  linearBuffer[j] = linerBufferOrigen[j];
           linearBuffer += canvas->width();
           linerBufferOrigen += bitmap->width();
      }
 }

/*int GcanvasPainter::drawBitmap(GgraphicsBitmap *bitmap, Grect &rect){
      Grect newRect = *(Grect*)bitmap;
      newRect.move(*(Grect*)canvas);
      newRect &= *(Grect*)canvas;
      bitmap->drawRect.setwidth(newRect.width());
      bitmap->drawRect.setheight(newRect.height());
      if (newRect.isEmpty()) {ShowMessage("Error");return 0;}
      int *linearBuffer = canvas->getLine(bitmap->top()) + bitmap->left();
      int *linerBufferOrigen = bitmap->getLine(0);
      for (int i=0;i<newRect.height();i++){
        for (int j=0;j<newRect.width();j++)
                  linearBuffer[j] = linerBufferOrigen[j];
           linearBuffer += canvas->width();
           linerBufferOrigen += bitmap->width();
      }
 }*/


int GcanvasPainter::drawText(Grect &rect,Gfont *font, string text, Gcolor color, TextAlignment textAlignment){
   Grect newRect = rect;
   newRect.move(*(Grect*)canvas);
   newRect &= *(Grect*)canvas;

    if (newRect.isEmpty()) return 0;

    Grect rectAlign = Grect(rect.left(),0,canvas->width(),canvas->height());

    int  maxWidth = (text.Lenght()+1)*(font->width + font->spaceSeparator);
    if (maxWidth < newRect.width()) newRect.setwidth(maxWidth);

    switch(textAlignment){
          case alignCenter:{
            newRect.alignCenter(rectAlign);
            break;
          }
          case alignCenterTop:{
            newRect.alignCenterHorizontal(*(Grect*)canvas);
            break;
          }
           case alignCenterBottom:{
            newRect.alignBottom(*(Grect*)canvas);
            newRect.alignCenterHorizontal(*(Grect*)canvas);
            break;
          }
          default: break;
    };

    bool wordWrap = false;
    char *ptr = text.c_str();
    int spaceSeparator = font->spaceSeparator;
    int  cwidth = newRect.width() / (font->width + spaceSeparator);
    int  cheight = newRect.height() / (font->height + spaceSeparator);
    int px = newRect.x();
    int py = newRect.y();
    int x = 0,y=0;
    while (*ptr) {
        if (x < cwidth) {
            iputc(px, py, color,*font, *ptr);
            px += font->width + spaceSeparator;
        };
        x++; ptr++;
        if (x ==  cwidth && wordWrap == true) {
            y++;
            if (y == cheight) return 0;
            px = newRect.x() + spaceSeparator;
            x=0;
            py += font->height + spaceSeparator;
        }
    }
 }


 void GcanvasPainter::iputc(int x, int y, Gcolor color, Gfont &font ,const char c) {
    int *linearBuffer = canvas->getLine(y) + x;
    int *fontMap = &font.linearBuffer[c*font.height];
    int newColor = color.valColor();
    char i, j,k;
    for (i = 0; i < font.height; i++){
        for (j = 0,k = font.width ; j < font.width ; j++,k--)
          if  (fontMap[i] & (1 << k))
               linearBuffer[j] = newColor;
           linearBuffer += canvas->width();
        }
}


int GcanvasPainter::backgroundColor(Gcolor color){
    int newColor = color.valColor();
    int *linearBuffer =  canvas->getGraphicsBitmap();
    for (int i=0;i< canvas->width()*canvas->height();i++)
         linearBuffer[i] = newColor;
}


void GcanvasPainter::upLines(int index, int countLines, int top) {
    if (countLines + index >= canvas->height() || top >= canvas->height())
        return;
    else {

        int indexBuffer = index * canvas->width();
        int maxPixels = countLines * canvas->width();
        int sizeBuffer = top * canvas->width();
        int *linearBuffer =  canvas->getGraphicsBitmap();
        for (int i = indexBuffer, j = indexBuffer + maxPixels; i < sizeBuffer; i++, j++) {
            linearBuffer[i] = linearBuffer[j];
        }
    }
}

void GcanvasPainter::downLines(int index, int countLines, int top) {
    if (countLines + index >= canvas->height())
        return;
    else {
        int indexBuffer = index * canvas->width();
        int maxPixels = countLines * canvas->width();
        int sizeBuffer = top * canvas->width();
        int *linearBuffer =  canvas->getGraphicsBitmap();
        for (int i = sizeBuffer, j = sizeBuffer - maxPixels; i > indexBuffer; i--, j--) {
            linearBuffer[i] = linearBuffer[j];
        }
    }
}

#pragma package(smart_init)

