//---------------------------------------------------------------------------


#pragma hdrstop
#include "control.h"
#include "Gbutton.h"
#include "Unit1.h"

//-------
Gcontrol::Gcontrol(int left,int top,int width,int height, Gcontrol *parent) : GgraphicsBitmap(width,height){
   canvas = new GcanvasPainter(this);       //Se crea la opcion de pintar sobre el
   visible = true;
   this->left = left;
   this->top = top;
   realRect = Grect(left,top,width,height);
   drawRect =  realRect;
   this->parent = parent;
   if (this->parent)
      parent->insertChildControl(this);
   focus = false;
   onClickDown = 0;
   onClickUp = 0;
   onFocused = 0;

}

Gcontrol::Gcontrol() : GgraphicsBitmap(200,200){
   canvas = new GcanvasPainter(this);       //Se crea la opcion de pintar sobre el
   this->left = 0;
   this->top = 0;
   realRect = Grect(0,0,200,200);
   drawRect =  realRect;
   parent = 0;
}
//------


//----------
void Gcontrol::resizeControl(int width,int height){
     resizeBitmap(width,height);
     realRect.setwidth(width);
     realRect.setheight(height);
     setwidth(width);
     setheight(height);
     rebuildControl();
}

void Gcontrol::setvisible(bool visible){
      if (this->visible == true && visible == false){
          clearToParent(drawRect,0);
          this->visible = false;
      }else if (this->visible == false && visible == true){
          this->visible = true;
          refresh();
      }
};

//-----
void Gcontrol::setleft(int left){
    if (left == this->left) return;
    clearToParent(drawRect,0);
    if (left < 0)                    //Si es menor que 0 el left relativo es 0 antes de moverse al espacio del padre
        realRect.setx(0);
    else
        realRect.setx(left);
      //Fixme : obtimizar solo la recta de intersect
     rebuildControl();             //Se dibuja el control

    this->left = left;
}


//-----
void Gcontrol::settop(int top){
    if (top == this->top) return;
     clearToParent(drawRect,0);
    if (top < 0)            //Si es menor que 0 el left relativo es 0 antes de moverse al espacio del padre
        realRect.sety(0);
    else
        realRect.sety(top);
     //Fixme : obtimizar solo la recta de intersect
     rebuildControl();             //Se dibuja el control
    this->top = top;
}


//--------
void Gcontrol::alingTo(controlAlign alignToParent){
    if (parent){
        switch(alignToParent){
            case alignNone :{
            break;
            };
        default: break;
        }
    }
}

//----------

int Gcontrol::insertChildControl(Gcontrol *childControl){
    int index = controls.indexOf(childControl);
    if (index != -1) return -1;
    childControl->zindex = controls.add(childControl);	    //zindex es inicialmente la posicion por orden de insercion
    childControl->parent = this;
    childControl->containerlayout =  containerlayout;       //Se toma el mismo layout que el padre
    childControl->rebuildControl();                         //Se reconstruye el control segun su posicion dentro del padre
    return childControl->zindex;
}

//------------- ok------------------------------------------------
void Gcontrol::rebuildControl(){
     if (parent){                                     //Es visible el componente
         drawRect  = realRect;                       //Se obtiene la recta original del componenete dentro del padre
         drawRect.move(parent->drawRect);            //Se mueve a la posicion relativa del padre
         drawRect &= parent->drawRect;               //Se queda con la interseccion
      if (drawRect.isEmpty()) {ShowMessage("Error");return;}
         moveToRelativeParent();
     }
}


//----------ok--------------------------------------------
void Gcontrol::rebuildControlsChilds(){
     Gcontrol *child;
     for (int i=0;i< controls.count();i++){
          child =  (Gcontrol*)parent->controls.getItem(i);
          child->rebuildControl();
          if (child->controls.count())
              child->rebuildControlsChilds();
     }
}

//----------ok-----------------------------------------------------------------
void Gcontrol::moveToRelativeParent(){
     Gcontrol *child;
     for (int i=0;i< controls.count();i++){
          child =  (Gcontrol*)controls.getItem(i);
          child->drawRect =  child->realRect;
          child->drawRect.move(drawRect);
          child->rebuildControlsChilds();
     }
}//-------

void Gcontrol::drawRectToLayout(Grect rect){
    Grect rectToPaint = drawRect & rect;
    int left = rect.x() -  drawRect.x();
    if (left < 0) left = 0;
    int top =  rect.y() -   drawRect.y();
    if (top < 0) top =0;
    containerlayout->drawBitmap(this, rectToPaint, left , top);
   // Form1->kpaintArea(this,rectToPaint) ;
}

//--------ok----------------------------------------------------------------
void Gcontrol::repaintZone(Grect rect, int index){
     Gcontrol *child;
     if (parent){
     for (int i=0;i< parent->controls.count();i++){                           //Se revisan todos los componentes hermanos
          child =  (Gcontrol*)parent->controls.getItem(i);
          if (child != this && child->zindex > index){                        //Si el hermano tiene prioridad para mostrarse sobre el, entonces
              if (child->drawRect.intersects(rect)){                          //Si se intersectan hay que pintar este pedazo
                  child->drawRectToLayout(rect);
                  if (child->controls.count()){
                      Gcontrol *subChild  = (Gcontrol*)child->controls.getItem(0);
                      subChild->drawRectToLayout(rect);
                      subChild->repaintZone(rect, subChild->zindex);
                 }
           }
      }
      }
    }
}

//--------------ok
void Gcontrol::clearToParent(Grect rect, int index){
    containerlayout->drawBitmap(parent, rect,realRect.x(),realRect.y());
    Form1->kpaintArea(parent,rect) ;
    repaintZone(rect, 0);
}
//-------ok-----------------------------------
void Gcontrol::setzindex(int zindex){
     if (this->zindex != zindex){
         this->zindex = zindex;
         refresh();
     }
}
//------------------------------------------

//-------ok----------------------------------
int Gcontrol::removeControl(Gcontrol* control){
   if (control->parent == this){
       control->clearToParent(control->drawRect,0);
   }
   return   controls.remove(control);
}
//---------------------------------------------


Gcontrol* Gcontrol::getControl(Gcontrol *control, int x, int y) {
    Gcontrol *selectedControl=0, *aux;
    int index = 0;
    for (int i = 0; i < control->controls.count(); i++) {
        aux = (Gcontrol*) control->controls.getItem(i);
        if (aux->drawRect.constains(x, y) && (aux->zindex > index)  && aux->visible) {
            index = aux->zindex;
            selectedControl = aux;
        }
    }
    if (selectedControl){
    if (selectedControl->controls.count() )
        selectedControl = getControl(selectedControl, x, y);
        return selectedControl;
   }
    return control;
}

//---ok------------------------------
void Gcontrol::refresh(){
      if (visible == true){
      containerlayout->drawBitmap(this, drawRect, 0, 0);
      Gcontrol *childControl;
       for (int i = 0; i < controls.count(); i++) {
        childControl = (Gcontrol*)controls.getItem(i);
        childControl->refresh();
       }
    repaintZone(drawRect, zindex);
    Form1->kpaintArea(this,drawRect) ;
    }
}

void Gcontrol::refresh(Grect rect){
   if (visible == true){
     rect.move(drawRect);
     drawRectToLayout(rect);
     Gcontrol *childControl;
       for (int i = 0; i < controls.count(); i++) {
        childControl = (Gcontrol*)controls.getItem(i);
        childControl->refresh();
       }
    repaintZone(drawRect, zindex);
    Form1->kpaintArea(this,rect) ;
   }
}

#pragma package(smart_init)










